/**
 * Create new actor.
 */
function Actor()
{
    this.x = 0;
    this.y = 0;
    this.direction = 0;
    this.radius = 25;

    this.speed = 0.0;
    this.rotation = 0.0;
}

/**
 * Check that the actor is focused or not.
 */
Actor.prototype.isFocused = function()
{
    var dx = this.x - mousePosition["x"];
    var dy = this.y - mousePosition["y"];
    var distance = Math.sqrt(dx * dx + dy * dy);
    if (distance < this.radius) {
        return true;
    }
    else {
        return false;
    }
}

/**
 * Render the actor.
 */
Actor.prototype.render = function(context)
{
    var radius = 20;

    context.save();

    context.translate(this.x, this.y);

    // NOTE: The (PI / 2) is required because the model coordinates are rotated!
    context.rotate(this.direction + Math.PI / 2);

    context.beginPath();
    context.arc(0, 0, radius, 0, 2 * Math.PI, false);
    context.fillStyle = '#888888';
    context.fill();
    context.lineWidth = 5;
    context.strokeStyle = '#444444';
    context.stroke();
    context.closePath();

    radius = 25;

    context.beginPath();
    context.moveTo(0, radius);
    context.lineTo(radius * -0.866025403784439, radius * -0.5);
    context.lineTo(radius * 0.866025403784439, radius * -0.5);
    context.lineTo(0, radius);
    if (this.isFocused() == false) {
        context.fillStyle = '#FF0000';
    }
    else {
        context.fillStyle = '#FF4444';
    }
    context.fill();
    context.strokeStyle = '#000000';
    context.lineWidth = 3;
    context.stroke();
    context.closePath();

    context.beginPath();
    context.moveTo(0, -12);
    context.lineTo(10, 0);
    context.lineTo(5, 0);
    context.lineTo(5, 8);
    context.lineTo(-5, 8);
    context.lineTo(-5, 0);
    context.lineTo(-10, 0);
    context.lineTo(0, -12);
    if (this.isFocused() == false) {
        context.fillStyle = '#FFCCCC';
    }
    else {
        context.fillStyle = '#FFFFFF';
    }
    context.fill();
    context.closePath();

    context.restore();
}

/**
 * Set the position of the actor.
 */
Actor.prototype.setPosition = function(x, y)
{
    this.x = x;
    this.y = y;
}

/**
 * Set the direction of the actor.
 */
Actor.prototype.setDirection = function(angle)
{
    this.direction = angle;
}

/**
 * Set the speed of the actor.
 */
Actor.prototype.setSpeed = function(speed)
{
    this.speed = speed;
}

/**
 * Set the rotation of the actor.
 */
Actor.prototype.setRotation = function(rotation)
{
    this.rotation = rotation;
}
