/**
 * Construct new tokenizer object.
 */
function CrossTokenizer()
{
}

/**
 * Set the text for tokenization.
 */
CrossTokenizer.prototype.setText = function(text)
{
    // The source text.
    this.text = text;

    // Index of the next character in the source.
    this.index = 0;

    // Store the actual cursor position.
    this.row = 1;
    this.column = 1;
}

/**
 * Get the next token.
 */
CrossTokenizer.prototype.getNextToken = function()
{
    this.skipWhiteSpaces();
    token = this.readToken();
    return token;
}

/**
 * Skip the whitespaces in the text.
 */
CrossTokenizer.prototype.skipWhiteSpaces = function()
{
    var isValid = false;
    do {
        var c = this.readNextCharacter();
        if (this.isComment(c)) {
            while (c != '\n') {
                c = this.getNextCharacter();
            }
        }
        else if (this.isWhiteSpace(c)) {
            this.getNextCharacter();
        }
        else {
            isValid = true;
        }
    } while (isValid == false);
}

/**
 * Is the character is whitespace?
 */
CrossTokenizer.prototype.isWhiteSpace = function(c)
{
    if (c == ' ' || c == '\n' || c == '\t') {
        return true;
    }
    return false;
}

/**
 * Is the next character is the beginning of the comment?
 */
CrossTokenizer.prototype.isComment = function(c)
{
    if (c == '#') {
        return true;
    }
    return false;
}

/**
 * Read the next token.
 */
CrossTokenizer.prototype.readToken = function()
{
    if (this.hasNextCharacter() == false) {
        return {"type": "empty"};
    }

    var c = this.readNextCharacter();
    if (c == '"') {
        return this.readLiteral();
    }
    else if (c == '{' || c == '}') {
        this.getNextCharacter();
        return {"type": "bracket", "value": c};
    }
    else if (this.isNumericChar(c)) {
        return this.readNumber();
    }
    else {
        return this.readKeyword();
    }
}

/**
 * Checks that the character is numeric.
 */
CrossTokenizer.prototype.isNumericChar = function(c)
{
    if ((c >= '0' && c <= '9') || c == '-' || c == '.') {
        return true;
    }
    return false;
}

/**
 * Read keyword token.
 */
CrossTokenizer.prototype.readKeyword = function()
{
    var value = "";
    do {
        value += this.getNextCharacter();
    } while (this.hasNextCharacter() && this.isKeywordChar(this.readNextCharacter()));
    return {"type": "keyword", "value": value};
}

/**
 * Is the character is valid keyword character?
 */
CrossTokenizer.prototype.isKeywordChar = function(c)
{
    if (c == '\n' || c == ' ' || c == '"' || c == '\t') {
        return false;
    }
    else {
        return true;
    }
}

/**
 * Read number token.
 */
CrossTokenizer.prototype.readNumber = function()
{
    var value = "";
    do {
        value += this.getNextCharacter();
    } while (this.hasNextCharacter() && this.isNumericChar(this.readNextCharacter()));
    return {"type": "number", "value": value};
}

/**
 * Read literal token.
 */
CrossTokenizer.prototype.readLiteral = function()
{
    // Skip the first " character.
    this.getNextCharacter();

    var value = "";
    while (this.hasNextCharacter() && this.readNextCharacter() != '"') {
        value += this.getNextCharacter();
    }
    if (this.index == this.text.length) {
        throw {"message": "Unexpected end of string literal!"};
    }

    // Skip the last " character.
    this.getNextCharacter();

    return {"type": "literal", "value": value};
}

/**
 * Check that there is at least one further character.
 */
CrossTokenizer.prototype.hasNextCharacter = function()
{
    if (this.index < this.text.length) {
        return true;
    }
    else {
        return false;
    }
}

/**
 * Read the next character.
 */
CrossTokenizer.prototype.readNextCharacter = function()
{
    var character = this.text[this.index];
    return character;
}

/**
 * Get the next character.
 */
CrossTokenizer.prototype.getNextCharacter = function()
{
    var character = this.text[this.index];

    if (character == '\n') {
        ++this.row;
        this.column = 1;
    }
    else {
        ++this.column;
    }

    ++this.index;
    return character;
}

/**
 * Get the current position.
 */
CrossTokenizer.prototype.getPosition = function()
{
    return {"row": this.row, "column": this.column};
}

